{----------------------------------------------------------------------------}
{                                                                            }
{ Program:    MAILBLT.PAS                                                    }
{ Purpose:    Generate Bulletin for use on NET/ECHO mail systems             }
{ Language:   Borland's Turbo Pascal Version 3.01                            }
{                                                                            }
{ Author:     Carl M. Evans                                                  }
{             Vervan's War Board BBS                                         }
{             P.O. Box 1176, Cucamonga, CA 91730  Voice: 714-989-7824        }
{             2400/1200/300 Baud (8-N-1 only)      Data: 714-989-7596        }
{                                                                            }
{ Notice:     This program is placed in the public domain and may be         }
{             freely used by anyone for any purpose.  When using this        }
{             code, please give credit to the original author and to         }
{             anyone who has subsequently improved or modified it.           }
{                                                                            }
{ Narrative:  This Turbo Pascal program simply fetches the current system    }
{             date and time from DOS and writes out a pure ASCII text file   }
{             which can be used on a BBS to report when the last net/echo    }
{             mail transfer took place. The report is automatically centered.}
{                                                                            }
{             As for "installation", it is suggested that you place this     }
{             program on your DOS path and invoke it as a part of the batch  }
{             file that you use to process your net/echo mail.               }
{                                                                            }
{             When you run this program, it expects to find a single command }
{             line parameter which is the name of the type of net/echo mail  }
{             system that is to be inserted in the output file. The default  }
{             is 'NET/ECHO'. There are no other inputs required. The output  }
{             file is hard coded to be MAIL.XXX in the current directory.    }
{                                                                            }
{ History:    Version 1.0, 11/25/88 - Original by Carl M. Evans              }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{----------------------------------------------------------------------------}
{.PA}
Program MAILBLT;
{$U-}
{$R+}
{$C-}
{============================================================================}
{ Declarations                                                               }
{============================================================================}

Const
  OutputFileDef  = 'MAIL.XXX';
  MailNameDef    = 'NET/ECHO';
  VersionNumber  = '1.0';

Type
  String2        = String[2];
  String4        = String[4];
  String5        = String[5];
  String10       = String[10];
  String25       = String[25];
  String80       = String[80];
  String128      = String[128];
  FileNameType   = String[80];

Type
  Regpack        = Record
    Case Integer of
      1 : (AX,BX,CX,DX,BP,SI,DI,DS,ES,Flags : Integer);
      2 : (AL,AH,BL,BH,CL,CH,DL,DH : Byte);
    End;

Var
{ Variables used for the output file                                         }
  OutputFile     : Text;
  OutputFileName : FileNameType;
  MailName       : String25;           { Name of Net/Echo Mail system        }

{ Miscellaneous Variables                                                    }
  ErrorCode,
  Len            : Integer;
  Temp           : String128;

{============================================================================}
{ Function to "pad" a date/time number to a 2-byte string                    }
{============================================================================}
Function Pad2(NumS:String2) :String2;

Begin { Pad2 }
  Case Length(NumS) of
    0            : Pad2 := '00';
    1            : Pad2 := '0' + NumS;
    2            : Pad2 := NumS;
  Else             Begin
                     Pad2[1] := NumS[1];
                     Pad2[2] := NumS[2];
                   End;
  End; {endcase}
End;  { Pad2  }

{============================================================================}
{ Function to "pad" a string with trailing spaces to a specified length      }
{============================================================================}
Function PadStrR(InpStr: String80; StrLen: Integer): String80;

Var
  I,
  Len            : Integer;
  TempString     : String80;

Begin { PadStrR }
  Len := Length(InpStr);
  If Len > StrLen then begin
    TempString := Copy(InpStr,1,StrLen);
  End else begin
    TempString := InpStr;
    For I := (Len + 1) to StrLen do TempString := TempString + ' ';
  End; {endif}
  PadStrR := TempString;
End;  { PadStrR }

{============================================================================}
{ Function to "pad" a string with leading spaces to a specified length       }
{============================================================================}
Function PadStrL(InpStr: String80; StrLen: Integer): String80;

Var
  I              : Integer;
  TempString     : String80;

Begin { PadStrL }
   If Length(InpStr) > StrLen then begin
      PadStrL := Copy(InpStr,1,StrLen);
   End else begin
      TempString := InpStr;
      For I := Length(TempString)+1 to StrLen do Insert(' ',TempString,1);
      PadStrL := TempString;
   End; {endif}
End;  { PadStrL }

{============================================================================}
{ Function to trim leading blanks from a string                              }
{============================================================================}
Function TrimL(InpStr: String80): String80;

Var
  I,
  Len            : Integer;

Begin { TrimL }
  Len := Length(InpStr);
  I := 1;
  While (I <= Len) and (InpStr[I] = ' ') do
    I := I + 1;
    TrimL := Copy(InpStr,I,Len-I+1)
End;  { TrimL }

{============================================================================}
{ Function to trim trailing blanks from a string                             }
{============================================================================}
Function TrimR(InpStr: String80): String80;

Var
  I              : Integer;

Begin { TrimR }
  I := Length(InpStr);
  While (I >= 1) and (InpStr[I] = ' ') do
    I := I - 1;
    TrimR := Copy(InpStr,1,I)
End;  { TrimR }

{============================================================================}
{ Function to center a string in a specified field length                    }
{============================================================================}
Function Center(InpStr: String80; StrLen: Integer): String80;

Var
  Spaces         : Integer;
  TempString     : String80;

Begin { Center }
   TempString := TrimR(TrimL(InpStr));
   Spaces     := (StrLen - Length(TempString) + 1) div 2;
   Center     := PadStrR(PadStrL(TempString,StrLen-Spaces),StrLen);
End;  { Center }

{============================================================================}
{ Function to convert a string to upper case characters                      }
{============================================================================}
Function UpperCase(InpStr: String80): String80;

Var
  I              : Integer;

Begin { UpperCase }
   For I := 1 to Length(InpStr) do
      UpperCase[I] := UpCase(InpStr[I]);
   UpperCase[0]  := InpStr[0]
End;  { UpperCase }

{============================================================================}
{ Function to get the current time of day from DOS                           }
{============================================================================}
Function TimeString: String5;

Var
  Reg            : Regpack;            {assign record}
  AH,
  AL,
  CH,
  CL,
  DH             : Byte;
  Hour,
  Min            : String2;

Begin { TimeString }
  With Reg do begin
    AH := $2C;                         {initialize correct registers}
    AX := AH shl 8 + AL;
  End; {endwith}
  Intr($21,Reg);                       {call interrupt}
  With Reg do begin
    Str(CX shr 8,Hour);                {convert to string}
    Str(CX mod 256,Min);                      { " }
  End; {endwith}
  TimeString := Pad2(Hour) + ':' + Pad2(Min);
End;  { TimeString }

{============================================================================}
{ Function to get the current DATE from DOS (MM-DD-YYYY format)              }
{============================================================================}
Function DateString : String10;

Var
  Reg            : Regpack;               {record for MsDos call}
  Month,
  Day            : String2;
  Year           : String4;
  DX,
  CX             : Integer;

Begin { DateString }
  With Reg do begin
    AX := $2A shl 8;
  End; {endwith}
  MsDos(Reg);                       { call function }
  With Reg do begin
    Str(CX,Year);                   {convert to string}
    Str(DX mod 256,Day);                    { " }
    Str(DX shr 8,Month);                    { " }
  End; {endwith}
  DateString := Pad2(Month) + '-' + Pad2(Day) + '-' + Year;
End; { DateString }

{============================================================================}
{ Procedure to make an annoying BEEP sound                                   }
{============================================================================}
Procedure Beep;

Begin { Beep }
  Sound(450);
  Delay(500);
  NoSound;
End;  { Beep }

{============================================================================}
{ Procedure to display command line syntax for sysop when error occurs       }
{============================================================================}
Procedure FatalError;

Begin { FatalError }
  Beep;
  Writeln(' ');
  Writeln('COMMAND LINE PARAMETER (optional) -');
  Writeln(' The default is:  NET/ECHO');
  Writeln(' You can override this default name by including any string,');
  Writeln(' up to 25 characters long as the first parameter.');
  Writeln(' ');
  Writeln(' For Example: MAILBLT PCBRELAY');
  Writeln(' ');
  HALT(1);
End;  { FatalError }

{============================================================================}
{ Procedure to parse the command line                                        }
{============================================================================}
Procedure ParseCommandLine;

Begin { ParseCommandLine }
  If ParamCount > 0 then begin
    Temp   := ParamStr(1);
    Temp   := TrimL(TrimR(Temp));
    Temp   := UpperCase(Temp);
    If Length(Temp) > 25 then begin
      MailName := Copy(Temp,1,25);
    End else MailName := Temp;
  End else begin
    MailName   := MailNameDef;
  End; {endif}
End;  { ParseCommandLine }

{============================================================================}
{ Procedure to create the OUTPUT file (MAIL.XXX)                             }
{============================================================================}
Procedure CreateOutputFile;

Var
  ReportStr      : String80;

Begin { CreateOutputFile }
  OutputFileName  := OutputFileDef;
  Assign(OutputFile,OutputFileName);
  {$I-} Rewrite(OutputFile); {$I+}
  ErrorCode := IOresult;
  If ErrorCode <> 0 then begin
    Close(OutputFile);
    Writeln(' ');
    Writeln('ERROR: Could not open ' + OutputFilename + ' for output!');
    FatalError;
  End; {endif}

{ Center the banner for the output report }
  ReportStr := Concat(' The last ',MailName,' mail transfer was at ',
    TimeString,' on ',DateString);
  Temp := ReportStr;
  If Length(ReportStr) > 78 then begin
    FatalError;
  End else ReportStr := Center(Temp,78);

{ Write the report to the output file }
  Writeln(OutputFile,'');
  Writeln(OutputFile,'[============================================================================]');
  Writeln(OutputFile,ReportStr);
  Writeln(OutputFile,'[============================================================================]');
  Writeln(OutputFile,'');
  Close(OutputFile);
End;  { CreateOutputFile }

{============================================================================}
{ Main Program                                                               }
{============================================================================}

Begin
  Writeln('');
  Writeln('MAIL-BLT Version ' + VersionNumber);
  Writeln('Bulletin Generator for Net/Echo Mail Systems');
  Writeln('Another Quality Program From Vervan' + chr(39) +
          's War Board (714) 989-7596');
  Writeln('');
  ParseCommandLine;
  CreateOutputFile;
End.

{===[ END OF PROGRAM ]=======================================================}
